import("CoreLibs/animation.lua")
import("CoreLibs/animator.lua")
import("CoreLibs/easing.lua")
import("CoreLibs/graphics.lua")
import("CoreLibs/object.lua")
import("CoreLibs/timer.lua")
import("CoreLibs/ui.lua")
import("CoreLibs/utilities/where.lua")

import("../lib/tiny-debug.lua")
import("../lib/tiny.lua")

local tiny <const> = tiny
world = tiny.world()

import("tiny-tools.lua")
import("assets.lua")
import("systems/filter-types.lua")
import("systems/gravity.lua")
import("systems/move-toward.lua")
import("systems/velocity.lua")
import("systems/spawner.lua")
import("systems/rounds.lua")
import("systems/camera-pan.lua")
import("systems/collision-resolution.lua")
import("systems/collision-detection.lua")

import("systems/draw.lua")
import("systems/input.lua")
import("systems/menu.lua")

import("ingredients.lua")
import("cart.lua")
import("utils.lua")

local gfx <const> = playdate.graphics
playdate.display.setRefreshRate(50)
gfx.setBackgroundColor(gfx.kColorWhite)

cart = Cart.new()
local floorSize = { x = 10000, y = 10 }
floor = {
    position = { x = 0, y = 235 },
    size = floorSize,
    canBeCollidedBy = 1 | 2 | 4,
    canBounce = {
        flat = { x = 0, y = 0 },
        mult = { x = 0.9, y = -0.5 },
    },
    drawAsRectangle = { size = floorSize },
    isSolid = true,
    sparkOnCollision = true,
}

Score = {
    points = 0,
}

-- TODO: Shops with random upgrades instead of fixed ones.

function Score:add(add)
    self.points = self.points + add
end

function Score:draw()
    gfx.drawText("Z|" .. self.points, 360, 5)
end

world:addEntity(floor)
world:addEntity(cart)
addAllSpawners(world)

world:addEntity(Ingredients.getFirst())

-- TODO: Re-enable when cart stops
playdate.setAutoLockDisabled(true)

local startMsOffset = -playdate.getCurrentTimeMilliseconds()

function playdate.update()
    local deltaSeconds = playdate.getElapsedTime()
    playdate.resetElapsedTime()
    gfx.clear(gfx.kColorWhite)
    playdate.drawFPS(5, 5)
    local fps = playdate.getFPS()
    if fps > 0 and fps < 20 then
        local currentTime = playdate.getCurrentTimeMilliseconds() + startMsOffset
        print("At " .. (currentTime / 1000) .. "s, FPS below 20: " .. fps)
    end

    floor.position.x = Camera.pan.x - 600

    world:update(math.min(1 / 40, deltaSeconds))

    gfx.setDrawOffset(0, 0)
    Score:draw()
end
