local MoveToward = {
    target = T.XyPair,
    range = T.number,
    speed = T.number,
}

local sqrt, abs = math.sqrt, math.abs

---@param xy1 XyPair
---@param xy2 XyPair
local function normalizeVector(xy1, xy2)
    local x = xy1.x - xy2.x
    local y = xy1.y - xy2.y
    local distance = sqrt((x * x) + (y * y))
    return x / distance, y / distance, distance
end

moveTowardSystem = filteredSystem("moveToward", { moveToward = MoveToward, position = T.XyPair }, function(e, dt, system)
    local xNorm, yNorm, distance = normalizeVector(e.position, e.moveToward.target)
    if distance > e.moveToward.range then
        return
    end

    -- TODO May be incorrect when signs are mismatched between vel and diff
    local xVel = xNorm * e.moveToward.speed * dt
    if abs(e.position.x - e.moveToward.target.x) < abs(xVel) then
        e.position.x = e.moveToward.target.x
    else
        e.position.x = e.position.x + xVel
    end

    local yVel = yNorm * e.moveToward.speed * dt
    if abs(e.position.y - e.moveToward.target.y) < abs(yVel) then
        e.position.y = e.moveToward.target.y
    else
        e.position.y = e.position.y + yVel
    end
end)
